Type: Package
Package: OlinkAnalyzeVignettes
Title: Vignettes for Analyzing Data using 'OlinkAnalyze'
Version: 1.0.1
Authors@R: c(
    person("Kathleen", "Nevola", , "biostattools@olink.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5183-6444", "kathy-nevola")),
    person("Marianne", "Sandin", role = "aut",
           comment = c(ORCID = "0000-0001-6186-963X", "marisand")),
    person("Jamey", "Guess", role = "aut",
           comment = c(ORCID = "0000-0002-4017-0923", "jrguess")),
    person("Simon", "Forsberg", role = "aut",
           comment = c(ORCID = "0000-0002-7451-9222", "simfor")),
    person("Christoffer", "Cambronero", role = "aut",
           comment = c("Orbmac")),
    person("Pascal", "Pucholt", role = "aut",
           comment = c(ORCID = "0000-0003-3342-1373", "AskPascal")),
    person("Boxi", "Zhang", role = "aut",
           comment = c(ORCID = "0000-0001-7758-6204", "boxizhang")),
    person("Masoumeh", "Sheikhi", role = "aut",
           comment = c("MasoumehSheikhi")),
    person("Klev", "Diamanti", role = "aut",
           comment = c(ORCID = "0000-0002-4922-8415", "klevdiamanti")),
    person("Amrita", "Kar", role = "aut",
           comment = c("amrita-kar")),
    person("Lei", "Conze", role = "aut", 
            comment = c("leiliuC")),
    person("Kristyn", "Chin", role = "aut",
            comment = c("kristynchin-olink")),
    person("Danai", "Topouza", role = "aut",
            comment = c("dtopouza", ORCID = "0000-0002-6897-9281")),
    person("Stephen", "Pollo", role = "aut",
            comment = c("spollo-olprot", ORCID = "0000-0001-9252-4976")),
    person("Kang", "Dong", role = "aut",
            comment = c("KangD-dev", ORCID = "0000-0002-4567-5007")),
    person("Kristian", "Hodén", role = "ctb",
           comment = c(ORCID = "0000-0003-0354-0662", "kristianHoden")),
    person("Per", "Eriksson", role = "ctb",
           comment = c(ORCID = "0000-0001-7633-403X", "b_watcher")),
    person("Nicola", "Moloney", role = "ctb",
            comment = c(ORCID = "0000-0003-4967-3284")),
    person("Britta", "Lötstedt", role = "ctb",
            comment = c(ORCID = "0000-0003-3545-5489")),
    person("Emmett", "Sprecher", role = "ctb",
           comment = c(ORCID = "0000-0002-7710-695X")),
    person("Jessica", "Barbagallo", role = "ctb",
           comment = c("jbarbagallo")),
    person("Olof", "Mansson", role = "ctr",
           comment = c("olofmansson")),
    person("Ola", "Caster", role = "ctb",
           comment = c("OlaCaster")),
    person("Olink", role = c("cph", "fnd"))
  )
Description: Exemplifying analysis of large-scale protein
    data from the 'Olink platform', primarily relative protein expression
    data that has been exported from
    'Olink NPX Software', as well as QUANT data from 'Olink'.
    QUANT data is log-transformed. Materials focus on reading data, 
    demonstrating data wrangling and quality control analysis,
    performing statistical analysis and generating
    figures to visualize the results of the statistical analysis. 
    The goal of this package is to guide users extract biological insights 
    from large-scale protein data run on the 'Olink platform'.
    More information on 'Olink' data can be found at <https://olink.com/>.
Contact: biostattools@olink.com
URL: https://olink.com/
        https://github.com/Olink-Proteomics/OlinkRPackage
License: AGPL (>= 3)
Encoding: UTF-8
Depends: R (>= 4.1.0)
Suggests: dplyr (>= 1.2.0), ggplot2, ggpubr, ggrepel, kableExtra,
        knitr, OlinkAnalyze, stringr, tidyr
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-01 18:27:06 UTC; kathy.nevola
Author: Kathleen Nevola [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5183-6444>, kathy-nevola),
  Marianne Sandin [aut] (ORCID: <https://orcid.org/0000-0001-6186-963X>,
    marisand),
  Jamey Guess [aut] (ORCID: <https://orcid.org/0000-0002-4017-0923>,
    jrguess),
  Simon Forsberg [aut] (ORCID: <https://orcid.org/0000-0002-7451-9222>,
    simfor),
  Christoffer Cambronero [aut] (Orbmac),
  Pascal Pucholt [aut] (ORCID: <https://orcid.org/0000-0003-3342-1373>,
    AskPascal),
  Boxi Zhang [aut] (ORCID: <https://orcid.org/0000-0001-7758-6204>,
    boxizhang),
  Masoumeh Sheikhi [aut] (MasoumehSheikhi),
  Klev Diamanti [aut] (ORCID: <https://orcid.org/0000-0002-4922-8415>,
    klevdiamanti),
  Amrita Kar [aut] (amrita-kar),
  Lei Conze [aut] (leiliuC),
  Kristyn Chin [aut] (kristynchin-olink),
  Danai Topouza [aut] (dtopouza, ORCID:
    <https://orcid.org/0000-0002-6897-9281>),
  Stephen Pollo [aut] (spollo-olprot, ORCID:
    <https://orcid.org/0000-0001-9252-4976>),
  Kang Dong [aut] (KangD-dev, ORCID:
    <https://orcid.org/0000-0002-4567-5007>),
  Kristian Hodén [ctb] (ORCID: <https://orcid.org/0000-0003-0354-0662>,
    kristianHoden),
  Per Eriksson [ctb] (ORCID: <https://orcid.org/0000-0001-7633-403X>,
    b_watcher),
  Nicola Moloney [ctb] (ORCID: <https://orcid.org/0000-0003-4967-3284>),
  Britta Lötstedt [ctb] (ORCID: <https://orcid.org/0000-0003-3545-5489>),
  Emmett Sprecher [ctb] (ORCID: <https://orcid.org/0000-0002-7710-695X>),
  Jessica Barbagallo [ctb] (jbarbagallo),
  Olof Mansson [ctr] (olofmansson),
  Ola Caster [ctb] (OlaCaster),
  Olink [cph, fnd]
Maintainer: Kathleen Nevola <biostattools@olink.com>
Repository: CRAN
Date/Publication: 2026-06-08 14:50:26 UTC
Built: R 4.6.0; ; 2026-06-08 18:47:14 UTC; unix
